# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from unittest import skipIf

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.core import mail
from django.test.utils import override_settings

from model_mommy import mommy
from registration.models import RegistrationProfile

from balafon.unit_tests import TestCase
from balafon.Crm import models


@skipIf(not ("balafon.Profile" in settings.INSTALLED_APPS), "registration not installed")
@override_settings(BALAFON_REGISTRATION_ENABLED=True)
class RegisterTestCase(TestCase):

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    def test_register(self):
        site = Site.objects.get_current()
        subscription_type1 = mommy.make(models.SubscriptionType, site=site)
        subscription_type2 = mommy.make(models.SubscriptionType, site=site)
        subscription_type3 = mommy.make(models.SubscriptionType, site=site)
        subscription_type4 = mommy.make(models.SubscriptionType)

        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'subscription_types': [subscription_type1.id, subscription_type2.id],
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        self.assertEqual(
            rp.user.contactprofile.subscriptions_ids,
            "{0},{1}".format(subscription_type1.id, subscription_type2.id)
        )

        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, data['gender'])

        self.assertEqual(
            True,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type1).accept_subscription
        )

        self.assertEqual(
            True,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type2).accept_subscription
        )

        self.assertEqual(
            False,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type3).accept_subscription
        )

        self.assertEqual(
            0,
            models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type4).count()
        )

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    @override_settings(BALAFON_REGISTRATION_ENABLED=False)
    def test_register_disable(self):
        site = Site.objects.get_current()
        subscription_type1 = mommy.make(models.SubscriptionType, site=site)
        subscription_type2 = mommy.make(models.SubscriptionType, site=site)
        subscription_type3 = mommy.make(models.SubscriptionType, site=site)
        subscription_type4 = mommy.make(models.SubscriptionType)

        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'subscription_types': [subscription_type1.id, subscription_type2.id],
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 404)

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    def test_register_no_subscription(self):
        site = Site.objects.get_current()

        subscription_type1 = mommy.make(models.SubscriptionType, site=site)

        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'subscription_types': [],
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        self.assertEqual(rp.user.contactprofile.subscriptions_ids, "")

        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, data['gender'])

        self.assertEqual(
            False,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type1).accept_subscription
        )

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    def test_register_one_subscription(self):
        site = Site.objects.get_current()

        subscription_type1 = mommy.make(models.SubscriptionType, site=site)
        subscription_type2 = mommy.make(models.SubscriptionType, site=site)

        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'subscription_types': [subscription_type1.id],
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        self.assertEqual(rp.user.contactprofile.subscriptions_ids, "{0}".format(subscription_type1.id))

        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, data['gender'])

        self.assertEqual(
            True,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type1).accept_subscription
        )

        self.assertEqual(
            False,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type2).accept_subscription
        )

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    def test_activate_invalid_subscription(self):
        site = Site.objects.get_current()

        subscription_type1 = mommy.make(models.SubscriptionType, site=site)
        subscription_type2 = mommy.make(models.SubscriptionType, site=site)

        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'subscription_types': [subscription_type1.id],
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        self.assertEqual(rp.user.contactprofile.subscriptions_ids, "{0}".format(subscription_type1.id))

        rp.user.contactprofile.subscriptions_ids = "bla"
        rp.user.contactprofile.save()

        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, data['gender'])

        self.assertEqual(
            False,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type1).accept_subscription
        )

        self.assertEqual(
            False,
            models.Subscription.objects.get(contact=contact, subscription_type=subscription_type2).accept_subscription
        )

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
    def test_register_no_gender(self):
        url = reverse('registration_register')
        data = {
            'email': 'john@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        self.assertEqual(rp.user.contactprofile.subscriptions_ids, "")
        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, 0)

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    def test_register_with_very_long_email(self):
        url = reverse('registration_register')
        data = {
            'email': ('a'*30)+'@toto.fr',
            'password1': 'PAss',
            'password2': 'PAss',
            'accept_termofuse': True,
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(email=data['email'])

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        rp = RegistrationProfile.objects.all()[0]
        self.assertEqual(rp.user, user)
        activation_url = reverse('registration_activate', args=[rp.activation_key])

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [data['email']])
        self.assertTrue(mail.outbox[0].body.find(activation_url))

        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        contact = models.Contact.objects.get(email=data['email'])
        self.assertEqual(contact.gender, data['gender'])

        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))

    def test_register_no_password(self):
        url = reverse('registration_register')
        data = {
            'email': 'toto@toto.fr',
            'password1': '',
            'password2': '',
            'accept_termofuse': True,
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

    def test_register_different_passwords(self):
        url = reverse('registration_register')
        data = {
            'email': 'toto@toto.fr',
            'password1': 'ABC',
            'password2': 'DEF',
            'accept_termofuse': True,
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

    def test_register_existing_email(self):
        user = mommy.make(models.User, email='toto@toto.fr')
        url = reverse('registration_register')
        data = {
            'email': 'toto@toto.fr',
            'password1': 'ABC',
            'password2': 'ABC',
            'accept_termofuse': True,
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.exclude(id=user.id).filter(email=data['email']).count(), 0)

    def test_register_existing_email_different_cases(self):
        user = mommy.make(models.User, email='toto@toto.fr')
        url = reverse('registration_register')
        data = {
            'email': 'TOTO@TOTO.FR',
            'password1': 'ABC',
            'password2': 'ABC',
            'accept_termofuse': True,
            'gender': 1,
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.exclude(id=user.id).filter(email=data['email']).count(), 0)



# -*- coding: utf-8 -*-
"""a simple store"""

from __future__ import unicode_literals

from rest_framework import permissions


def get_staff_api_permissions():
    """get public api permissions"""
    return [permissions.IsAdminUser]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0015_auto_20170120_1218'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='subscriptiontype',
            options={'ordering': ('order_index',), 'verbose_name': 'Subscription type', 'verbose_name_plural': 'Subscription types'},
        ),
        migrations.AddField(
            model_name='subscriptiontype',
            name='order_index',
            field=models.IntegerField(default=0, verbose_name='order index'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomMenu',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=100, verbose_name='label')),
                ('icon', models.CharField(default=b'', max_length=20, verbose_name='icon', blank=True)),
                ('order_index', models.IntegerField(default=0)),
                ('position', models.IntegerField(default=0, help_text='Where the menu will be set', verbose_name='position', choices=[(0, 'Menu'), (1, 'Planning')])),
            ],
            options={
                'ordering': ['order_index', 'label'],
                'verbose_name': 'Custom menu',
                'verbose_name_plural': 'Custom menus',
            },
        ),
        migrations.CreateModel(
            name='CustomMenuItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=100, verbose_name='label')),
                ('icon', models.CharField(default=b'', max_length=20, verbose_name='icon', blank=True)),
                ('url', models.CharField(default=b'', max_length=100, verbose_name='url', blank=True)),
                ('reverse', models.CharField(default=b'', max_length=100, verbose_name='reverse', blank=True)),
                ('order_index', models.IntegerField(default=0)),
                ('attributes', models.CharField(default=b'', max_length=100, verbose_name='attributes', blank=True)),
                ('only_for_users', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='only for users', blank=True)),
                ('parent', models.ForeignKey(to='Users.CustomMenu')),
            ],
            options={
                'ordering': ['order_index', 'label'],
                'verbose_name': 'Custom menu item',
                'verbose_name_plural': 'Custom menu items',
            },
        ),
        migrations.CreateModel(
            name='Favorite',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField(verbose_name='object id')),
                ('content_type', models.ForeignKey(related_name='user_favorite_set', verbose_name='content_type', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(related_name='user_favorite_set', verbose_name='user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Favorite',
                'verbose_name_plural': 'Favorites',
            },
        ),
        migrations.CreateModel(
            name='UserHomepage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(verbose_name='URL')),
                ('user', models.OneToOneField(verbose_name='user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User homepage',
                'verbose_name_plural': 'User homepages',
            },
        ),
        migrations.CreateModel(
            name='UserPreferences',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('notify_due_actions', models.BooleanField(default=False, verbose_name='Notify due actions')),
                ('message_in_favorites', models.BooleanField(default=False, verbose_name='Create automatically a favorite for message posted from the public form')),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User preferences',
                'verbose_name_plural': 'User preferences',
            },
        ),
        migrations.AlterUniqueTogether(
            name='favorite',
            unique_together=set([('user', 'content_type', 'object_id')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""admin"""

from __future__ import unicode_literals

from django.contrib import admin
from django.contrib.messages import success, error
from django.utils.translation import ugettext_lazy as _

import floppyforms.__future__ as forms

from balafon.widgets import VerboseManyToManyRawIdWidget
from balafon.Crm import models
from balafon.Crm.forms.actions import ActionMenuAdminForm
from balafon.Crm.settings import get_language_choices


class HasParentFilter(admin.SimpleListFilter):
    """filter items to know if they have a parent"""
    title = _('Has parent')
    parameter_name = 'has_parent'

    def lookups(self, request, model_admin):
        return [
            (1, _('Yes')),
            (2, _('No')),
        ]

    def queryset(self, request, queryset):
        value = self.value()
        if value == '1':
            return queryset.filter(parent__isnull=False)
        elif value == '2':
            return queryset.filter(parent__isnull=True)
        return queryset


@admin.register(models.Zone)
class ZoneAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'parent', 'type']
    ordering = ['type', 'name']
    list_filter = ['type', HasParentFilter, 'parent']
    search_fields = ['name']


@admin.register(models.EntityRole)
class EntityRoleAdmin(admin.ModelAdmin):
    pass


@admin.register(models.SameAs)
class SameAsAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['priority_contact', 'other_contacts', 'contacts_count']



@admin.register(models.OpportunityType)
class OpportunityTypeAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ActionMenu)
class ActionMenuAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['action_type', 'view_name', 'label', 'icon', 'a_attrs', 'order_index', 'only_for_status_str']
    list_filter = ['action_type', ]
    list_editable = ['order_index', ]
    search_fields = ['label', ]
    form = ActionMenuAdminForm


@admin.register(models.EntityType)
class EntityTypeAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['id', 'name', 'gender', 'order', 'subscribe_form']
    list_editable = ['name', 'gender', 'order', 'subscribe_form']
    list_filter = ['subscribe_form']


@admin.register(models.ZoneType)
class ZoneTypeAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'type']
    ordering = ['type', 'name']


def initialize_status2(modeladmin, request, queryset):
    for action_type in queryset:
        if action_type.default_status2:
            actions_queryset = action_type.action_set.filter(status2__isnull=True)
            actions_count = actions_queryset.count()
            actions_queryset.update(status2=action_type.default_status2)
            success(
                request,
                _("initialize {0} actions of type '{1}' with status2 '{2}'").format(
                    actions_count, action_type.name, action_type.default_status2.name
                )
            )
        else:
            error(
                request,
                _("No default status2 for actions type '{0}'").format(action_type.name)
            )
initialize_status2.short_description = _("Initialize status2 to default if Null")


def reset_status2(modeladmin, request, queryset):
    for action_type in queryset:
        if action_type.default_status2:
            actions_queryset = action_type.action_set.filter(status2=action_type.default_status2)
            actions_count = actions_queryset.count()
            actions_queryset.update(status2=None)
            success(
                request,
                _("reset {0} actions of type '{1}'").format(
                    actions_count, action_type.name
                )
            )
        else:
            error(
                request,
                _("No default status2 for actions type '{0}'").format(action_type.name)
            )
reset_status2.short_description = _("Reset status2 to Null")


def set_action_previous_status(modeladmin, request, queryset):
    for action_type in queryset:
        if not action_type.track_status:
            actions_queryset = action_type.action_set.all()
            actions_count = actions_queryset.count()
            for action in actions_queryset:
                action.previous_status = action.status
                action.save()
            success(
                request,
                _("set previous values for {0} actions").format(actions_count)
            )
        else:
            error(
                request,
                _("{0} : track status should be disabled when executing this action").format(action_type.name)
            )
set_action_previous_status.short_description = _("Track status : Set previous status")


def create_action_initial_track(modeladmin, request, queryset):
    for action_type in queryset:
        if action_type.track_status:
            actions_queryset = action_type.action_set.all()
            actions_count = actions_queryset.count()
            ignored_count = 0
            for action in actions_queryset:
                if action.status and action.status.is_final and action.done_date and action.done_date:
                    track_datetime = action.done_date
                else:
                    track_datetime = action.planned_date

                no_tracks_yet = models.ActionStatusTrack.objects.filter(action=action).count() == 0
                if action.status and track_datetime and no_tracks_yet:
                    models.ActionStatusTrack.objects.create(
                        status=action.status,
                        action=action,
                        datetime=track_datetime
                    )
                else:
                    ignored_count += 1

            success(
                request,
                _("create tracks for {0} actions / {1} ignored").format(actions_count - ignored_count, ignored_count)
            )
        else:
            error(
                request,
                _("{0} : track status should be enabled when executing this action").format(action_type.name)
            )
create_action_initial_track.short_description = _("Track status : create initial track")


@admin.register(models.ActionType)
class ActionTypeAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = [
        'name', 'set', 'status_defined', 'subscribe_form', 'action_number',
        'default_template', 'is_editable', 'hide_contacts_buttons', 'track_status', 'is_default'
    ]
    list_filter = [
        'set', 'subscribe_form', 'number_auto_generated', 'number_generator', 'default_template', 'action_template',
        'hide_contacts_buttons', 'track_status',
    ]
    list_editable = ['set', 'subscribe_form', 'hide_contacts_buttons', ]
    actions = [initialize_status2, reset_status2, set_action_previous_status, create_action_initial_track]
    search_fields = ['name']


@admin.register(models.OpportunityStatus)
class OpportunityStatusAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'ordering']


class SubscriptionInline(admin.TabularInline):
    """custom inline"""
    model = models.Subscription


@admin.register(models.Contact)
class ContactAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['lastname', 'firstname', 'entity', 'email']
    search_fields = ['lastname', 'email']
    raw_id_admin = ('entity',)
    inlines = (SubscriptionInline,)

    def get_form(self, *args, **kwargs):
        form_class = super(ContactAdmin, self).get_form(*args, **kwargs)
        class custom_form_class(form_class):
            def __init__(self, *args, **kwargs):
                super(custom_form_class, self).__init__(*args, **kwargs)
                self.fields['favorite_language'].widget = forms.Select(choices=get_language_choices())
        return custom_form_class


@admin.register(models.Group)
class GroupAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'description', 'subscribe_form']
    search_fields = ['name']
    list_filter = ['subscribe_form']
    list_editable = ['subscribe_form']
    filter_horizontal = ['entities', 'contacts']


class GroupInline(admin.TabularInline):
    """custom inline"""
    model = models.Group


@admin.register(models.City)
class CityAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'parent']
    search_fields = ['name']
    ordering = ['name']
    list_filter = [HasParentFilter, 'parent', ]
    raw_id_fields = ('groups',)


@admin.register(models.Entity)
class EntityAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ('name', 'type')
    search_fields = ['name']


@admin.register(models.Opportunity)
class OpportunityAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', ]
    search_fields = ['name', ]
    raw_id_admin = ('entity',)


@admin.register(models.Action)
class ActionAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = [
        'id', 'planned_date', 'created', 'subject', 'type', 'status', 'done', 'opportunity', 'number', 'amount'
    ]
    search_fields = ['subject', 'number']
    list_filter = ['type', 'status', 'done', 'opportunity']
    readonly_fields = ['created', 'modified', 'created_by', 'last_modified_by']
    date_hierarchy = 'created'
    raw_id_fields = ['contacts', 'entities',]

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in self.raw_id_fields:
            kwargs['widget'] = VerboseManyToManyRawIdWidget(db_field.rel, self.admin_site)
        else:
            return super(ActionAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        kwargs.pop('request')
        return db_field.formfield(**kwargs)


@admin.register(models.CustomFieldChoice)
class CustomFieldChoicedAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['value', 'label', 'order']


@admin.register(models.CustomField)
class CustomFieldAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'model', 'label', 'widget', 'ordering', 'import_order', 'export_order']
    list_editable = ['ordering', 'import_order', 'export_order']
    list_filter = ('model', 'widget')
    filter_horizontal = ['choices']


@admin.register(models.EntityCustomFieldValue)
class EntityCustomFieldValueAdmin(admin.ModelAdmin):
    """custom admin view"""
    search_fields = ['entity']


@admin.register(models.ContactCustomFieldValue)
class ContactCustomFieldValueAdmin(admin.ModelAdmin):
    """custom admin view"""
    search_fields = ['contact']


@admin.register(models.ContactsImport)
class ContactsImportAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ActionSet)
class ActionSetAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'ordering']
    list_editable = ['ordering']
    search_fields = ['name']


@admin.register(models.ActionStatus)
class ActionStatusAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'ordering', 'is_final', 'background_color', 'fore_color']
    list_filter = ['is_final']
    list_editable = ['ordering', 'is_final', 'background_color', 'fore_color']
    search_fields = ['name']


@admin.register(models.ActionDocument)
class ActionDocumentAdmin(admin.ModelAdmin):
    pass


@admin.register(models.RelationshipType)
class RelationshipTypeAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['id', 'name', 'reverse']
    list_editable = ['name', 'reverse']


@admin.register(models.Relationship)
class RelationshipAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['id', 'contact1', 'relationship_type', 'contact2']


@admin.register(models.SubscriptionType)
class SubscriptionTypeAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'site', 'order_index']
    list_editable = ['order_index']


@admin.register(models.TeamMember)
class TeamMemberAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['name', 'user', 'active']
    list_filter = ['active']


@admin.register(models.StreetType)
class StreetTypeAdmin(admin.ModelAdmin):
    pass


@admin.register(models.MailtoSettings)
class MailtoSettingsAdmin(admin.ModelAdmin):
    """custom admin view"""
    list_display = ['action_type', 'subject', 'bcc']


@admin.register(models.ActionStatusTrack)
class ActionStatusTrackAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ActionNumberGenerator)
class ActionNumberGeneratorAdmin(admin.ModelAdmin):
    list_display = ('name', 'number', )



# -*- coding: utf-8 -*-
"""test email tracking"""

from __future__ import unicode_literals

from datetime import datetime, date

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import management
from django.core import mail
from django.core.urlresolvers import reverse

from model_mommy import mommy
from coop_cms.models import Newsletter

from balafon.Crm import models
from balafon.Emailing.models import Emailing
from balafon.Emailing.tests import BaseTestCase
from balafon.Emailing.utils import patch_emailing_html


class EmailTrackingTest(BaseTestCase):
    """Use a image to track opened emails"""

    def setUp(self):
        """before each test"""
        super(EmailTrackingTest, self).setUp()
        settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'
        settings.COOP_CMS_REPLY_TO = 'titi@toto.fr'

        settings.COOP_CMS_SITE_PREFIX = "toto.fr"
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

    def test_track_image(self):
        """make sure that opened emails are incremented for every contact reading the email"""

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, entity=entity, email=name+'@toto.fr', lastname=name.capitalize())
            for name in names
        ]

        content = '''
            <h2>Hello #!-fullname-!#!</h2>
            <p>
            Visit <a href="http://toto.fr">us</a><a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a>
            </p>
        '''

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SENT,
            scheduling_dt=datetime.now(),
            sending_dt=datetime.now()
        )
        for contact in contacts:
            emailing.sent_to.add(contact)
        emailing.save()

        self.assertEqual(emailing.opened_emails.count(), 0)

        for contact in contacts[:-1]:
            tracking_url = reverse("emailing_email_tracking", args=[emailing.id, contact.uuid])
            response = self.client.get(tracking_url)
            self.assertEqual(response.status_code, 200)

        self.assertEqual(emailing.opened_emails.count(), len(contacts)-1)
        for contact in contacts[:-1]:
            self.assertTrue(contact in list(emailing.opened_emails.all()))

        for contact in contacts[-1:]:
            self.assertFalse(contact in list(emailing.opened_emails.all()))

    def test_track_image_twice(self):
        """make sure that opened emails are not incremented twice for every contact reading the email"""

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, entity=entity, email=name+'@toto.fr', lastname=name.capitalize())
            for name in names
        ]

        content = '''
            <h2>Hello #!-fullname-!#!</h2>
            <p>
            Visit <a href="http://toto.fr">us</a> <a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a>
            </p>
        '''

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SENT,
            scheduling_dt=datetime.now(),
            sending_dt=datetime.now()
        )
        for contact in contacts:
            emailing.sent_to.add(contact)
        emailing.save()

        self.assertEqual(emailing.opened_emails.count(), 0)

        for contact in contacts[:-1]:
            tracking_url = reverse("emailing_email_tracking", args=[emailing.id, contact.uuid])
            response = self.client.get(tracking_url)
            self.assertEqual(response.status_code, 200)

            response = self.client.get(tracking_url)
            self.assertEqual(response.status_code, 200)

        self.assertEqual(emailing.opened_emails.count(), len(contacts)-1)
        for contact in contacts[:-1]:
            self.assertTrue(contact in list(emailing.opened_emails.all()))

        for contact in contacts[-1:]:
            self.assertFalse(contact in list(emailing.opened_emails.all()))

    def test_send_newsletter_check_tracking(self):
        """make sure that tracking url is set in email"""

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, entity=entity, email=name+'@toto.fr', lastname=name.capitalize())
            for name in names
        ]

        content = '''
            <h2>Hello #!-fullname-!#!</h2>
            <p>
            Visit <a href="http://toto.fr">us</a><a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a>
            </p>
        '''

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        #check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        for email, contact in zip(outbox, contacts):
            self.assertNotEqual(newsletter.get_site_prefix(), "")
            tracking_url = newsletter.get_site_prefix()
            tracking_url += reverse("emailing_email_tracking", args=[emailing.id, contact.uuid])
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(tracking_url) >= 0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Emailing', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='magiclink',
            options={'verbose_name': 'Magic link', 'verbose_name_plural': 'Magic links'},
        ),
        migrations.AlterField(
            model_name='emailing',
            name='lang',
            field=models.CharField(default=b'', max_length=5, verbose_name='language', blank=True, choices=[(b'', 'Par d\xe9faut'), (b'fr', 'Fran\xe7ais')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0010_clean_same_as'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='favorite_language',
            field=models.CharField(default=b'', max_length=10, verbose_name='favorite language', blank=True, choices=[(b'', 'Par d\xe9faut'), (b'fr', 'Fran\xe7ais')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""cities"""

from __future__ import unicode_literals

import json

from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponse

from balafon.Crm import models
from balafon.Crm.settings import get_default_country
from balafon.permissions import can_access
from balafon.utils import log_error


def get_city_name(request, city):
    """view"""
    #Anybody can accesss (public contact form of a website)
    try:
        city_id = int(city)
        city = models.City.objects.get(id=city_id)
        return HttpResponse(json.dumps({'name': city.name}), 'application/json')
    except ValueError:
        return HttpResponse(json.dumps({'name': city}), 'application/json')


@user_passes_test(can_access)
@log_error
def get_city_id(request):
    """view"""
    name = (request.GET.get('name'))
    country_id = request.GET.get('country', None)

    if country_id is not None:
        country_id = int(country_id)

    default_country = models.Zone.objects.get(name=get_default_country(), parent__isnull=True)

    if country_id is None:
        cities = models.City.objects.filter(name__iexact=name)
    else:
        if country_id == 0 or country_id == default_country.id:
            cities = models.City.objects.filter(name__iexact=name).exclude(parent__code='')
        else:
            cities = models.City.objects.filter(name__iexact=name, parent__id=country_id)

    if cities.count() != 1:
        city_id = name
    else:
        city_id = cities[0].id

    return HttpResponse(json.dumps({'id': city_id}), 'application/json')


def get_cities(request):
    """view"""
    #subscribe form : no login required
    term = request.GET.get('term')
    country_id = request.GET.get('country', None)
    if country_id == 'undefined':
        # javascript none
        country_id = 0

    if country_id is not None:
        country_id = int(country_id)

    default_country = models.Zone.objects.get(name=get_default_country(), parent__isnull=True)

    if country_id is None:
        cities_queryset = models.City.objects.filter(name__icontains=term)[:10]
    else:
        if country_id == 0 or country_id == default_country.id:
            cities_queryset = models.City.objects.filter(name__icontains=term).exclude(parent__code='')[:10]
        else:
            cities_queryset = models.City.objects.filter(name__icontains=term, parent__id=country_id)[:10]

    cities = [{'id': city.id, 'name': city.name} for city in cities_queryset]

    return HttpResponse(json.dumps(cities), 'application/json')
